--***************************************************************
-- Trailer Sound
--***************************************************************
source(Utils.getFilename("mogliBase20.lua", g_currentModDirectory))
mogliBase20.newClass( "TrailerSound", "" )
--***************************************************************

TrailerSoundGlobals = {}
TrailerSoundGlobals.volumeWheels        = 2
TrailerSoundGlobals.maxVolumeWheels     = 2
TrailerSoundGlobals.pitchWheels         = 1
TrailerSoundGlobals.pitchTrailer        = 1
TrailerSoundGlobals.moreTrailerFactor   = 1.2
TrailerSoundGlobals.maxVolumeTrailer    = 4
TrailerSoundGlobals.volumeTrailerField  = 4
TrailerSoundGlobals.volumeTrailerStreet = 1
TrailerSoundGlobals.indoorFactor        = 0.3
TrailerSoundGlobals.volumeBrake         = 1

local function initGlobals(createIfMissing)
	local file
	file = TrailerSound.baseDirectory.."zzzTrailerSound.xml"
	if fileExists(file) then	
		TrailerSound.globalsLoad( file, "TrailerSoundGlobals", TrailerSoundGlobals )	
	else
		print("ERROR: NO GLOBALS IN "..file)
	end
	
	file = TrailerSound.modsDirectory.."zzzTrailerSound.xml"
	if fileExists(file) then	
		TrailerSound.globalsLoad( file, "TrailerSoundGlobals", TrailerSoundGlobals )	
	elseif createIfMissing then
		TrailerSound.globalsCreate()
	end
end
initGlobals()

function TrailerSound:load(xmlFile)
	self.trailerSound          = {}
	self.trailerSound.count    = 0
	self.trailerSound.sumDt    = 0
	self.trailerSound.drawTime = 0
	self.trailerSound.brake    = false
	
	if self.isClient then
		local filename = Utils.getFilename( "trailerSound.wav" , TrailerSound.baseDirectory )
		self.trailerSound.trailerSample = createSample("trailerSound")
		loadSample(self.trailerSound.trailerSample, filename, false)
		setSampleVolume( self.trailerSound.trailerSample, 0 )
		setSamplePitch(  self.trailerSound.trailerSample, 0.5 )
		
		filename = Utils.getFilename( "surren.wav" , TrailerSound.baseDirectory )
		self.trailerSound.wheelSample = createSample("wheelSound")
		loadSample(self.trailerSound.wheelSample, filename, false)
		setSampleVolume( self.trailerSound.wheelSample, 0 )
		setSamplePitch(  self.trailerSound.wheelSample, 0.5 )
		
		filename = Utils.getFilename( "auflaufbremse.wav" , TrailerSound.baseDirectory )
		self.trailerSound.brakeSample = createSample("brakeSound")
		loadSample(self.trailerSound.brakeSample, filename, false)
		setSampleVolume( self.trailerSound.brakeSample, 1 )
		setSamplePitch(  self.trailerSound.brakeSample, 1 )
	end
end

function TrailerSound:delete()
	if self.trailerSound ~= nil then
		if self.trailerSound.trailerSample ~= nil then
			delete( self.trailerSound.trailerSample )			
		end
		if self.trailerSound.wheelSample ~= nil then
			delete( self.trailerSound.wheelSample )			
		end
		if self.trailerSound.brakeSample ~= nil then
			delete( self.trailerSound.brakeSample )			
		end
	end
end

function TrailerSound:updateTick(dt)	

	if      self:getIsActiveForSound()
			and TrailerSoundGlobals.volumeWheels > 0
			and TrailerSoundGlobals.pitchWheels  > 0 then

		self.trailerSound.sumDt = self.trailerSound.sumDt + dt
		if self.trailerSound.sumDt > 1000 then
			self.trailerSound.count = TrailerSound.countTrailers( self )
			self.trailerSound.sumDt = self.trailerSound.sumDt - 1000
		end			

		local volIndoor = 1
		for _,camera in pairs(self.cameras) do
			if camera.isInside and camera.isActivated then
				volIndoor = TrailerSoundGlobals.indoorFactor
				break
			end
		end		
		
		local kmh     = math.min( math.max( self.lastSpeed * self.speedDisplayScale * 3600, 0), 100 )	
		if kmh > 20 and TrailerSoundGlobals.pitchWheels > 0 then
			local c1,c2 = 0,0
			for i = 1, table.getn(self.wheels) do
				c2 = c2 + 1
				local x,_,z = getWorldTranslation( self.wheels[i].repr )
				if self.wheels[i].tireType ~= nil and not TrailerSound.isField( x, z ) then
					if     WheelsUtil.tireTypes[self.wheels[i].tireType].name == "mud"
							or WheelsUtil.tireTypes[self.wheels[i].tireType].name == "offRoad" then
						c1 = c1 + 1
					else
						c1 = c1 + 0.5 
					end
				end
			end
			
			if c1 > 0 and c2 > 0 then
				local vol = math.min( math.max( kmh - 20, 0 ) / 30, TrailerSoundGlobals.maxVolumeWheels )
				local ptc = 1 + vol
				vol = vol * c1 / c2
				
				
				if vol <= 0 then
					if isSamplePlaying( self.trailerSound.wheelSample ) then
						stopSample( self.trailerSound.wheelSample )
					end
				else
					if isSamplePlaying( self.trailerSound.wheelSample ) then
						setSampleVolume( self.trailerSound.wheelSample, vol * TrailerSoundGlobals.volumeWheels * volIndoor )
					else
						playSample( self.trailerSound.wheelSample, 1, vol * TrailerSoundGlobals.volumeWheels * volIndoor, 0 )
					end
					setSamplePitch(  self.trailerSound.wheelSample, ptc * TrailerSoundGlobals.pitchWheels )
				end
			elseif isSamplePlaying( self.trailerSound.wheelSample ) then
				stopSample( self.trailerSound.wheelSample )
			end
		elseif isSamplePlaying( self.trailerSound.wheelSample ) then
			stopSample( self.trailerSound.wheelSample )
		end
		
		if self.trailerSound.count > 0 and kmh >= 3 and TrailerSoundGlobals.pitchTrailer > 0 then
			if self.brakeLightsVisibility then
				if not self.trailerSound.brake then
					playSample( self.trailerSound.brakeSample, 1, TrailerSoundGlobals.volumeBrake * volIndoor, 0 )
					self.trailerSound.brake = true
				end
			elseif self.trailerSound.brake then
				self.trailerSound.brake = false
			end
	
			local vol = math.min( math.max( kmh / 50, 0 ), TrailerSoundGlobals.maxVolumeTrailer )
			local ptc = vol * vol
			local x,_,z = getWorldTranslation( self.components[1].node )
			if TrailerSound.isField( x, z ) then
				vol = vol * TrailerSoundGlobals.volumeTrailerField
			else
				vol = vol * TrailerSoundGlobals.volumeTrailerStreet
			end
			
			for i=2,self.trailerSound.count do
				vol = vol * TrailerSoundGlobals.moreTrailerFactor
			end
	
			if vol <= 0 then
				if isSamplePlaying( self.trailerSound.trailerSample ) then
				  stopSample( self.trailerSound.trailerSample )
				end
			else
				if isSamplePlaying( self.trailerSound.trailerSample ) then
					setSampleVolume( self.trailerSound.trailerSample, vol * volIndoor )
				else
					playSample( self.trailerSound.trailerSample, 1, vol * volIndoor, 0 )
				end
				setSamplePitch( self.trailerSound.trailerSample, ptc * TrailerSoundGlobals.pitchTrailer )
			end
		elseif isSamplePlaying( self.trailerSound.trailerSample ) then
			stopSample( self.trailerSound.trailerSample )
		end
	end
	
end

function TrailerSound:onLeave()
	if self.trailerSound ~= nil then
		if      self.trailerSound.trailerSample ~= nil 
				and isSamplePlaying( self.trailerSound.trailerSample ) then
			stopSample( self.trailerSound.trailerSample )
		end
		if      self.trailerSound.wheelSample ~= nil 
				and isSamplePlaying( self.trailerSound.wheelSample ) then
			stopSample( self.trailerSound.wheelSample )
		end
		if      self.trailerSound.brakeSample ~= nil 
				and isSamplePlaying( self.trailerSound.brakeSample ) then
			stopSample( self.trailerSound.brakeSample )
		end
	end
end

function TrailerSound.isField( x, z )
	for channel=0,3 do
		if Utils.getDensity(g_currentMission.terrainDetailId, channel, x-0.25, z-0.25, x+0.5, z, x, z+0.5 ) > 0 then
			return true
		end
	end
	
	id = g_currentMission.fruits[FruitUtil.FRUITTYPE_GRASS].id
  setDensityReturnValueShift(id, -1)
  local ret = getDensityParallelogram(id, x-0.25, z-0.25, 0.5, 0, 0, 0.5, 0, g_currentMission.numFruitStateChannels)
  setDensityReturnValueShift(id, 0)
	if ret > 0 then
		return true
	end
	
	return false
end

function TrailerSound:countTrailers()
	
	local count = 0
	if self == nil then
		return 0
	end
	if self.trailerSoundActive then
		count = 1
	end
	if self.attachedImplements ~= nil and table.getn(self.attachedImplements) > 0 then
		for _, v in pairs(self.attachedImplements) do
			count = count + TrailerSound.countTrailers( v.object )
		end	
  end	
	
	return count
end


